# coding: utf-8

"""
    Newsletter2Go-API (https://api.newsletter2go.com)

    <h5>JSON first</h5><br/>Our REST API exchanges data in the JSON data format. Every parameter you pass (with a few exceptions e.g. when you upload files) should therefore be formatted in JSON and our API will always return results in JSON as well.<br/><br/><h5>Very RESTful</h5><br/>Our API follows a very RESTful approach.<br/>Most importantly, we implemented the following four request methods for CRUD operations:<br/><br/>POST - Create a new record<br/>GET - Retrieve / read records without changing anything<br/>PATCH - Update an existing record<br/>DELETE - Delete one or more records<br/><br/><h5>HTTP Status codes</h5><br/>We also follow the most common HTTP status codes when outputting the API response:<br/><br/><b>2xx - Successful calls</b><br/>200 - Success<br/>201 - Created<br/><br/><b>4xx - Error codes</b><br/>400 - API error - inspect the body for detailed information about what went wrong. The most common error is \"code\":1062, which means, that a unique constraint was hit. For example if the name of the group is already in use.<br/>401 - Not authenticated - invalid access_token. Check if the access_token has expired or is incorrect.<br/>403 - Access denied<br/>404 - Call not available or no records found<br/><br/><b>5xx - API error - please contact support</b><br/><br/><h5>Response format</h5><br/>The API always returns data in the following format<br/><br/><code>{<br/>&nbsp;&nbsp;\"info\": {<br/>&nbsp;&nbsp;&nbsp;&nbsp;\"count\": 0<br/>&nbsp;&nbsp;},<br/>&nbsp;&nbsp;\"value\": [<br/>&nbsp;&nbsp;&nbsp;{<br/>&nbsp;&nbsp;&nbsp;&nbsp;\"key\": \"value\"<br/>&nbsp;&nbsp;&nbsp;}<br/>&nbsp;&nbsp;]<br/>}</code><br/><br/><h5>Recurring GET parameters</h5><br/><ul><li>_filter - a complex filter for filtering the result set based on <a target=\"blank\" href=\"https://tools.ietf.org/html/draft-nottingham-atompub-fiql-00\">FIQL</a>.</li><li>_limit - the maximum number records returned.</li><li>_offset - pagination for the result-set</li><li>_expand - submit true to get all default fields of the resource</li><li>_fields - submit a comma-separated list of case-sensetive field-names to get the values of these fields in the response. You can use this the get values of custom attribute of recipients for example.</li></ul><br/><br/><h5>Filter language</h5><br/>The filter language for filtering results is based on <a target=\"blank\" href=\"https://tools.ietf.org/html/draft-nottingham-atompub-fiql-00\">FIQL</a>.<br/>With the only restriction, that plain values must be surrounded by \". For example first_name==\"Max\"<br/>The following operators are supported<ul><li>Equals - <b>==</b></li><li>Not equals - <b>=ne=</b></li><li>Greater than - <b>=gt=</b></li><li>Greater than equals - <b>=ge=</b></li><li>Lower than - <b>=lt=</b></li><li>Lower than equals - <b>=le=</b></li><li>Like - <b>=like=</b> (in combination with % you are able to search for <i>starts with</i>, <i>ends with</i>, <i>contains</i>. For example first_name=like=\"%Max%\")</li><li>Not like - <b>=nlike=</b></li><li>Logical and - <b>;</b></li><li>Logical or - <b>,</b></li></ul><br/><br/><h5>How to make Calls?</h5><br/>After you authenticated and received a valid access_token, you have to pass it in every consecutive call. Use the Authorization header for that purpose as follows:<br/><code>xhr.setRequestHeader(\"Authorization\", \"Bearer \" + your_access_token);</code><br/><br/>Every call takes additional parameters that can be used to modify the request. These parameters should be passed as JSON<br/><code>var params = {<br/>&nbsp;&nbsp;\"key\"= \"value\"<br/>}<br/>xhr.send(JSON.stringify(params));</code><br/><br/><h5>Sending transactional emails</h5><br/>It is very important to understand the following concept in order for you to take full advantage of our powerful personalization features and our detailed reports when sending transactional emails.<br/><br/>First, you will have to create a new mailing. We recommend that you create that mailing through our UI in order to take full advantage of our powerful newsletter builder. This way, we will automatically create cross-client optimized and responsive HTML. Yet another advantage lies in the possibility for other users (e.g. the marketing team) to change the layout or the content of the mailing without having to contact the developer (you).<br/><br/>Of course, it is also possible to create a mailing through the API. When doing so, you can also take advantage of our cross-client optimized responsive HTML by passing us JSON or YAML according to the Newsletter2Go Markup Language.<br/>No matter how you create the mailing, try to create *one* reusable template. You can customize individual emails by inserting placeholders for personalized fields such as name, products or other information that will be filled through an API call when sending.<br/><br/>By only creating one template, you can take advantage of our full reporting since all emails will be treated part of a \"campaign\". When you change that template, we will create a new version of the mailing in the background and you will be able to see the difference in performance in the reports. This is particularily useful when you are trying to test and optimize different versions of transactional emails such as a sign up email.<br/><br/>After creating a mailing, you will have access to its ID. Use that ID to actually send the email in the next step.<br/><br/>When sending an email, you have to pass the newsletter ID and information about the recipient. Either pass the recipient ID or pass all the recipient's data (including the e-mail-address) as JSON.<br/><br/>If you only pass the recipient ID, we will use his or her data from your list to personalize the mailing. If you pass full recipient data as JSON, we will try to merge the data with existing data from your list.<br/><br/>You can also pass additional data such as product data which is not saved in your list. Just make sure that the placeholders in the source of the mailing correspond to the parameters that you are passing.<br/>This way you can also create for-loops which can be useful if you pass different amounts of data for each recipient (e.g. a purchase confirmation where you want to list all the products that were just bought).

    OpenAPI spec version: 1.0.0
    Contact: support@newsletter2go.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
    
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
    
        http://www.apache.org/licenses/LICENSE-2.0
    
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from __future__ import absolute_import

import sys
import os
import re

# python 2 and python 3 compatibility library
from six import iteritems

from ..configuration import Configuration
from ..api_client import ApiClient


class RecipientApi(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        config = Configuration()
        if api_client:
            self.api_client = api_client
        else:
            if not config.api_client:
                config.api_client = ApiClient()
            self.api_client = config.api_client

    def add_recipient_to_group(self, lid, gid, id, **kwargs):
        """
        add single recipient to group
        add Recipient to Group

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.add_recipient_to_group(lid, gid, id, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str lid: the id of the list (required)
        :param str gid: the id of the group (required)
        :param str id: the id of the recipient (required)
        :return: PatchResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['lid', 'gid', 'id']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_recipient_to_group" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'lid' is set
        if ('lid' not in params) or (params['lid'] is None):
            raise ValueError("Missing the required parameter `lid` when calling `add_recipient_to_group`")
        # verify the required parameter 'gid' is set
        if ('gid' not in params) or (params['gid'] is None):
            raise ValueError("Missing the required parameter `gid` when calling `add_recipient_to_group`")
        # verify the required parameter 'id' is set
        if ('id' not in params) or (params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `add_recipient_to_group`")


        resource_path = '/lists/{lid}/groups/{gid}/recipients/{id}'.replace('{format}', 'json')
        path_params = {}
        if 'lid' in params:
            path_params['lid'] = params['lid']
        if 'gid' in params:
            path_params['gid'] = params['gid']
        if 'id' in params:
            path_params['id'] = params['id']

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'POST',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='PatchResponse',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def add_recipients_to_group(self, lid, gid, **kwargs):
        """
        add all Recipients to the given group
        add all Recipients to the given group. Pay attention to the filter, if you forget to submit it, all recipients of the current list are getting added to the group!

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.add_recipients_to_group(lid, gid, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str lid: the id of the list (required)
        :param str gid: the id of the group (required)
        :param str filter: a FIQL-Filter
        :return: PatchResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['lid', 'gid', 'filter']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_recipients_to_group" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'lid' is set
        if ('lid' not in params) or (params['lid'] is None):
            raise ValueError("Missing the required parameter `lid` when calling `add_recipients_to_group`")
        # verify the required parameter 'gid' is set
        if ('gid' not in params) or (params['gid'] is None):
            raise ValueError("Missing the required parameter `gid` when calling `add_recipients_to_group`")


        resource_path = '/lists/{lid}/groups/{gid}/recipients'.replace('{format}', 'json')
        path_params = {}
        if 'lid' in params:
            path_params['lid'] = params['lid']
        if 'gid' in params:
            path_params['gid'] = params['gid']

        query_params = {}
        if 'filter' in params:
            query_params['_filter'] = params['filter']

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'POST',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='PatchResponse',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def create_recipient(self, recipient, **kwargs):
        """
        create new recipient(s) or updates existing ones
        create new recipient(s).<br>If a custom attribute shall by saved for the new recipient just submit them in the json body by name.<br>If <b>multiple recipients</b> shall by saved at once, submit multiple recipient-objects as a json array.<br><b>In this case all objects must share the same attributes.</b><br/>If the e-mail-address exists already the existing recipient gets updated.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.create_recipient(recipient, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param RecipientPost recipient: the data to save (required)
        :return: ApiResponsePost
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['recipient']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_recipient" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'recipient' is set
        if ('recipient' not in params) or (params['recipient'] is None):
            raise ValueError("Missing the required parameter `recipient` when calling `create_recipient`")


        resource_path = '/recipients'.replace('{format}', 'json')
        path_params = {}

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'recipient' in params:
            body_params = params['recipient']

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'POST',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='ApiResponsePost',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def delete_recipient(self, lid, id, **kwargs):
        """
        delete the recipient
        delete one recipient

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.delete_recipient(lid, id, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str lid: the id of the list (required)
        :param str id: the id of the recipient (required)
        :return: ApiResponseDelete
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['lid', 'id']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_recipient" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'lid' is set
        if ('lid' not in params) or (params['lid'] is None):
            raise ValueError("Missing the required parameter `lid` when calling `delete_recipient`")
        # verify the required parameter 'id' is set
        if ('id' not in params) or (params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `delete_recipient`")


        resource_path = '/lists/{lid}/recipients/{id}'.replace('{format}', 'json')
        path_params = {}
        if 'lid' in params:
            path_params['lid'] = params['lid']
        if 'id' in params:
            path_params['id'] = params['id']

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'DELETE',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='ApiResponseDelete',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def get_recipients(self, lid, **kwargs):
        """
        get all Recipients of selected list
        Get Recipients. If the API shall return custom attributes also, submit them in the _fields-Parameter by name.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.get_recipients(lid, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str lid: the id of the list (required)
        :param str filter: a FIQL-Filter
        :param int limit: a limit for list-responses
        :param int offset: an offset for list-responses
        :param bool expand: true if attributes should be returned or not
        :param list[str] fields: list of case-sensetive fields which should be returned.    Only needed if _expand is false or special attributes are needed
        :return: RecipientResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['lid', 'filter', 'limit', 'offset', 'expand', 'fields']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_recipients" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'lid' is set
        if ('lid' not in params) or (params['lid'] is None):
            raise ValueError("Missing the required parameter `lid` when calling `get_recipients`")


        resource_path = '/lists/{lid}/recipients'.replace('{format}', 'json')
        path_params = {}
        if 'lid' in params:
            path_params['lid'] = params['lid']

        query_params = {}
        if 'filter' in params:
            query_params['_filter'] = params['filter']
        if 'limit' in params:
            query_params['_limit'] = params['limit']
        if 'offset' in params:
            query_params['_offset'] = params['offset']
        if 'expand' in params:
            query_params['_expand'] = params['expand']
        if 'fields' in params:
            query_params['_fields'] = params['fields']

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'GET',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='RecipientResponse',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def get_recipients_by_group(self, lid, gid, **kwargs):
        """
        get all Recipients of selected group
        get Recipients

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.get_recipients_by_group(lid, gid, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str lid: the id of the list (required)
        :param str gid: the id of the group (required)
        :param str filter: a FIQL-Filter
        :param int limit: a limit for list-responses
        :param int offset: an offset for list-responses
        :param bool expand: true if attributes should be returned or not
        :param list[str] fields: list of case-sensetive fields which should be returned.    Only needed if _expand is false or special attributes are needed
        :return: RecipientResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['lid', 'gid', 'filter', 'limit', 'offset', 'expand', 'fields']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_recipients_by_group" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'lid' is set
        if ('lid' not in params) or (params['lid'] is None):
            raise ValueError("Missing the required parameter `lid` when calling `get_recipients_by_group`")
        # verify the required parameter 'gid' is set
        if ('gid' not in params) or (params['gid'] is None):
            raise ValueError("Missing the required parameter `gid` when calling `get_recipients_by_group`")


        resource_path = '/lists/{lid}/groups/{gid}/recipients'.replace('{format}', 'json')
        path_params = {}
        if 'lid' in params:
            path_params['lid'] = params['lid']
        if 'gid' in params:
            path_params['gid'] = params['gid']

        query_params = {}
        if 'filter' in params:
            query_params['_filter'] = params['filter']
        if 'limit' in params:
            query_params['_limit'] = params['limit']
        if 'offset' in params:
            query_params['_offset'] = params['offset']
        if 'expand' in params:
            query_params['_expand'] = params['expand']
        if 'fields' in params:
            query_params['_fields'] = params['fields']

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'GET',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='RecipientResponse',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def import_recipients_init(self, lid, file, **kwargs):
        """
        Initialize the import of recipients by file
        Initialize the import recipients by file

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.import_recipients_init(lid, file, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str lid: the id of the list (required)
        :param file file: recipient file to upload (required)
        :return: InlineResponse200
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['lid', 'file']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method import_recipients_init" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'lid' is set
        if ('lid' not in params) or (params['lid'] is None):
            raise ValueError("Missing the required parameter `lid` when calling `import_recipients_init`")
        # verify the required parameter 'file' is set
        if ('file' not in params) or (params['file'] is None):
            raise ValueError("Missing the required parameter `file` when calling `import_recipients_init`")


        resource_path = '/lists/{lid}/recipients/import/init'.replace('{format}', 'json')
        path_params = {}
        if 'lid' in params:
            path_params['lid'] = params['lid']

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}
        if 'file' in params:
            local_var_files['file'] = params['file']

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type(['multipart/form-data'])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'POST',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='InlineResponse200',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def import_recipients_save(self, lid, data, **kwargs):
        """
        Start the import recipients by file
        Start the import recipients by file. The returned id can be used to fetch further statistics about the import.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.import_recipients_save(lid, data, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str lid: the id of the list (required)
        :param Data data: the data to start the import (required)
        :return: ApiResponsePost
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['lid', 'data']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method import_recipients_save" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'lid' is set
        if ('lid' not in params) or (params['lid'] is None):
            raise ValueError("Missing the required parameter `lid` when calling `import_recipients_save`")
        # verify the required parameter 'data' is set
        if ('data' not in params) or (params['data'] is None):
            raise ValueError("Missing the required parameter `data` when calling `import_recipients_save`")


        resource_path = '/lists/{lid}/recipients/import/save'.replace('{format}', 'json')
        path_params = {}
        if 'lid' in params:
            path_params['lid'] = params['lid']

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'data' in params:
            body_params = params['data']

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'POST',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='ApiResponsePost',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def import_recipients_statistics(self, id, **kwargs):
        """
        Get statistics about the import by file
        Get statistics about the import by file.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.import_recipients_statistics(id, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str id: the id of the import job (required)
        :return: ImportResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['id']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method import_recipients_statistics" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'id' is set
        if ('id' not in params) or (params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `import_recipients_statistics`")


        resource_path = '/import/{id}/info'.replace('{format}', 'json')
        path_params = {}
        if 'id' in params:
            path_params['id'] = params['id']

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'GET',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='ImportResponse',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def remove_recipient_from_group(self, lid, gid, id, **kwargs):
        """
        remove single recipient from group
        remove Recipient to Group

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.remove_recipient_from_group(lid, gid, id, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str lid: the id of the list (required)
        :param str gid: the id of the group (required)
        :param str id: the id of the recipient (required)
        :return: PatchResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['lid', 'gid', 'id']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method remove_recipient_from_group" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'lid' is set
        if ('lid' not in params) or (params['lid'] is None):
            raise ValueError("Missing the required parameter `lid` when calling `remove_recipient_from_group`")
        # verify the required parameter 'gid' is set
        if ('gid' not in params) or (params['gid'] is None):
            raise ValueError("Missing the required parameter `gid` when calling `remove_recipient_from_group`")
        # verify the required parameter 'id' is set
        if ('id' not in params) or (params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `remove_recipient_from_group`")


        resource_path = '/lists/{lid}/groups/{gid}/recipients/{id}'.replace('{format}', 'json')
        path_params = {}
        if 'lid' in params:
            path_params['lid'] = params['lid']
        if 'gid' in params:
            path_params['gid'] = params['gid']
        if 'id' in params:
            path_params['id'] = params['id']

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'DELETE',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='PatchResponse',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def remove_recipients_from_group(self, lid, gid, **kwargs):
        """
        remove all Recipients from given group
        remove all Recipients from given group. Pay attention to the filter, if you forget to submit it, all recipients are getting removed from the group!

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.remove_recipients_from_group(lid, gid, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str lid: the id of the list (required)
        :param str gid: the id of the group (required)
        :param str filter: a FIQL-Filter
        :return: PatchResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['lid', 'gid', 'filter']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method remove_recipients_from_group" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'lid' is set
        if ('lid' not in params) or (params['lid'] is None):
            raise ValueError("Missing the required parameter `lid` when calling `remove_recipients_from_group`")
        # verify the required parameter 'gid' is set
        if ('gid' not in params) or (params['gid'] is None):
            raise ValueError("Missing the required parameter `gid` when calling `remove_recipients_from_group`")


        resource_path = '/lists/{lid}/groups/{gid}/recipients'.replace('{format}', 'json')
        path_params = {}
        if 'lid' in params:
            path_params['lid'] = params['lid']
        if 'gid' in params:
            path_params['gid'] = params['gid']

        query_params = {}
        if 'filter' in params:
            query_params['_filter'] = params['filter']

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'DELETE',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='PatchResponse',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def remove_recipients_from_list(self, lid, **kwargs):
        """
        remove all Recipients from selected list
        remove Recipients from selected list. Pay attention to the filter, if you forget to submit it, all recipients of the current list are getting removed!

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.remove_recipients_from_list(lid, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str lid: the id of the list (required)
        :param str filter: a FIQL-Filter
        :return: PatchResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['lid', 'filter']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method remove_recipients_from_list" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'lid' is set
        if ('lid' not in params) or (params['lid'] is None):
            raise ValueError("Missing the required parameter `lid` when calling `remove_recipients_from_list`")


        resource_path = '/lists/{lid}/recipients'.replace('{format}', 'json')
        path_params = {}
        if 'lid' in params:
            path_params['lid'] = params['lid']

        query_params = {}
        if 'filter' in params:
            query_params['_filter'] = params['filter']

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'DELETE',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='PatchResponse',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def subscribe_recipient(self, code, recipient, **kwargs):
        """
        Creates a new recipient and sends a DOI-Mail
        Creates a new recipient and sends the corresponding DOI-Mail attached to the previously created subscribe-form. The recipient will be available as soon as the recipient has confirmed the DOI-Mail.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.subscribe_recipient(code, recipient, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str code: the subscribe-code displayed when creating a subscribe-form (required)
        :param RecipientSubscribe recipient: the recipient to subscribe (required)
        :return: RecipientSubscribeResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['code', 'recipient']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method subscribe_recipient" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'code' is set
        if ('code' not in params) or (params['code'] is None):
            raise ValueError("Missing the required parameter `code` when calling `subscribe_recipient`")
        # verify the required parameter 'recipient' is set
        if ('recipient' not in params) or (params['recipient'] is None):
            raise ValueError("Missing the required parameter `recipient` when calling `subscribe_recipient`")


        resource_path = '/forms/submit/{code}'.replace('{format}', 'json')
        path_params = {}
        if 'code' in params:
            path_params['code'] = params['code']

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'recipient' in params:
            body_params = params['recipient']

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = []

        response = self.api_client.call_api(resource_path, 'POST',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='RecipientSubscribeResponse',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def update_recipient(self, lid, id, recipient, **kwargs):
        """
        update the recipient.
        update one Recipient. If a value of a custom attribute shall by changed just submit it in the json body

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.update_recipient(lid, id, recipient, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str lid: the id of the list (required)
        :param str id: the id of the recipient (required)
        :param RecipientPatch recipient: the data to update (required)
        :return: PatchResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['lid', 'id', 'recipient']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_recipient" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'lid' is set
        if ('lid' not in params) or (params['lid'] is None):
            raise ValueError("Missing the required parameter `lid` when calling `update_recipient`")
        # verify the required parameter 'id' is set
        if ('id' not in params) or (params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `update_recipient`")
        # verify the required parameter 'recipient' is set
        if ('recipient' not in params) or (params['recipient'] is None):
            raise ValueError("Missing the required parameter `recipient` when calling `update_recipient`")


        resource_path = '/lists/{lid}/recipients/{id}'.replace('{format}', 'json')
        path_params = {}
        if 'lid' in params:
            path_params['lid'] = params['lid']
        if 'id' in params:
            path_params['id'] = params['id']

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'recipient' in params:
            body_params = params['recipient']

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'PATCH',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='PatchResponse',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response

    def update_recipients(self, lid, **kwargs):
        """
        update all Recipients in selected list
        Updates all recipients in the given list where the filter applies. If a value of a custom attribute shall by changed just submit it in the json body. Pay attention to the filter, if you forget to submit it, all recipients of the current list are getting updated!

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.update_recipients(lid, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str lid: the id of the list (required)
        :param str filter: a FIQL-Filter
        :return: PatchResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['lid', 'filter']
        all_params.append('callback')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_recipients" % key
                )
            params[key] = val
        del params['kwargs']

        # verify the required parameter 'lid' is set
        if ('lid' not in params) or (params['lid'] is None):
            raise ValueError("Missing the required parameter `lid` when calling `update_recipients`")


        resource_path = '/lists/{lid}/recipients'.replace('{format}', 'json')
        path_params = {}
        if 'lid' in params:
            path_params['lid'] = params['lid']

        query_params = {}
        if 'filter' in params:
            query_params['_filter'] = params['filter']

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = ['OAuth']

        response = self.api_client.call_api(resource_path, 'PATCH',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='PatchResponse',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'))
        return response
